/*
 * file:        ADC.h
 * purpose:     class for analog digital conversion
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef ADC_H
#define ADC_H
#include "Pin.h"
#include <avr/io.h>

namespace srd {

#define NUM_OVERSAMPLING        1

class ADConverter
{
  struct Values {
    int16_t  v[8];
    int16_t  temp[NUM_OVERSAMPLING][8];
    int8_t   cOversampling;
    int8_t   activeChannel;
    int8_t   activeMuxes;
    int8_t   completed;
    };
  static volatile struct Values values;
  static void conversionComplete() __asm__(STR(ADC_vect)) __attribute__((__signal__, __used__, __externally_visible__));
  static void calcValues();
  inline static void setActiveChannel(int8_t channel) {
    ADMUX &= 0xE0;
    ADMUX |= channel & 0x1F;
    };

public:
  enum RefVoltage {
       AREF,
       AVCC,                                               // needs capacitor at AREF Pin
       Internal = 3                                        // needs capacitor at AREF Pin
       };
  enum Prescaler {
       PS_2 = 1,
       PS_4,
       PS_8,
       PS_16,
       PS_32,
       PS_64,
       PS_128
       };
  enum AutoTrigger {
       FreeRunning,
       AnalogComparator,
       ExtInterrupt,
       Timer0Compare,
       Timer0Overflow,
       Timer1CompareB,
       Timer1Overflow,
       Timer1Capture
       };
  ADConverter(RefVoltage reference, Prescaler prescale, int8_t leftAdjusted = 0);

  inline void setAutoTriggerMode(AutoTrigger trigger) { SFIOR &= ~(trigger << 5); SFIOR |= trigger; };
  inline void start() { ADCSRA |= 1 << ADEN | 1 << ADSC; }
  inline void stop()  { ADCSRA &= ~(1 << ADEN); }
  void reset();
  inline void enableAutoTrigger(int8_t enable = 1) {
    if (enable) ADCSRA |=   1 << ADATE;
    else        ADCSRA &= ~(1 << ADATE);
    };
  void enableChannel(Pin pin, int8_t enable = 1);
  void selectNextChannel();
  inline int16_t getValue(int8_t channel) const { return channel < 8 ? values.v[channel] : 0; }
  inline int8_t conversionCompleted() const { return values.completed; }
  };
};
#endif
