/*
 * file:        Buzzer.C
 * purpose:     define a class for using a buzzer with 8bit timer
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include "Buzzer.h"

using namespace srd;


Buzzer::Buzzer(const Pin& pin)
 : pin(pin) {
  this->pin.toOutput();
#ifdef __AVR_ATmega16__
  TCCR0 |= 1 << WGM01;                                     // ctc mode (mode 2 OCR0A top)
  TCCR0 |= 1 << COM00;                                     // toggle pin at compare
#endif
  }


Buzzer::~Buzzer() {
  }


void Buzzer::run() {
  if (duration > 0) {
     if (!--duration) stop();
     }
  }


void Buzzer::alarm(int8_t tone, int16_t duration) {
  this->tone = tone;
  this->toneLength = duration;
  this->duration = toneLength;
  start();
  }


void Buzzer::doBeep(int8_t tone, int16_t duration) {
#ifdef __AVR_ATmega16__
  OCR0 = tone;                                             // timer top == frequency
#endif
  this->tone = tone;
  this->toneLength = duration;
  this->duration = duration;
  start();
  }


void Buzzer::check4Repeat() {
  if (!duration) {
     if (repeat > 0) {
        --repeat;
        --duration;
        }
     }
  else if (duration < 0) {
     duration = toneLength;
     start();
     }
  }
