/*
 * file:        Buzzer.h
 * purpose:     declare a class for buzzer usage by 8bit timer
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef BUZZER_H
#define BUZZER_H
#include "Runnable.h"
#include "Pin.h"
#include "SystemTicker.h"

namespace srd {

class Buzzer : public Runnable
{
public:
  Buzzer(const Pin& pin);
  virtual ~Buzzer();
  virtual void run();

  void alarm(int8_t tone = ToneA7, int16_t duration = 1000);
  void doBeep(int8_t tone = ToneH4, int16_t duration = 300);
  void check4Repeat();

protected:
  enum Tones {                                             // closest timer tops for real frequencies
    ToneH4 = 253,
    ToneC5 = 238,
    ToneD5 = 212,
    ToneE5 = 189,
    ToneF5 = 178,
    ToneG5 = 159,
    ToneA5 = 141,
    ToneH5 = 126,
    ToneC6 = 118,
    ToneD6 = 105,
    ToneE6 = 94,
    ToneF6 = 88,
    ToneG6 = 79,
    ToneA6 = 70,
    ToneH6 = 62,
    ToneC7 = 59,
    ToneD7 = 52,
    ToneE7 = 46,
    ToneF7 = 44,
    ToneG7 = 39,
    ToneA7 = 34
    };
  inline void start() {
#ifdef __AVR_ATmega16__
    TCCR0 |= 1 << CS01 | 1 << CS00;                        // buzzer uses timer with prescaler 64
#endif
    };

public:
  inline void stop() {
#ifdef __AVR_ATmega16__
    TCCR0 &= ~(1 << CS00 | 1 << CS01 | 1 << CS02);
#endif
    };

private:
  Pin pin;
  int16_t toneLength;
  volatile int16_t duration;
  volatile int8_t repeat;
  int8_t tone;
  };
};
#endif
