/*
 * file:        Encoder.h
 * purpose:     class for usage of turning encoders (works together with system tick interrupt)
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef ENCODER_H
#define ENCODER_H
#include "Runnable.h"
#include "Pin.h"
#include "PushButton.h"

namespace srd {

class CounterHandler
{
public:
  virtual ~CounterHandler();
  virtual void handleCounter(int8_t delta);
  };

class DefaultCounterHandler : public CounterHandler
{
public:
  virtual ~DefaultCounterHandler();
  virtual void handleCounter(int8_t delta);

  int8_t counter;
  };


class Encoder : public Runnable
{
public:
  Encoder(const Pin& left, const Pin&right, const PushButton bt)
   : left(left)
   , right(right)
   , bt(bt)
  {
    this->left.toInput();
    this->right.toInput();
    handler = &defHandler;
    };
  virtual ~Encoder();
  virtual void run();

  PushButton* pushButton() { return &bt; }
  void setCounterHandler(CounterHandler* handler);

private:
  Pin left;
  Pin right;
  PushButton bt;
  DefaultCounterHandler defHandler;
  CounterHandler* handler;
  int8_t encLast;
  int8_t encDelta;
  };
};
#endif
