/*
 * file:        LCD.h
 * purpose:     class for LCD display handling using only 3 pins
 *              together with a shift register (i.e. 74*164)
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey, bit handling is based on the work of Peter Dannegger
 * copyright:   2008-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef LCD_H
#define LCD_H
#include "Base.h"
#include "Pin.h"
#include <util/delay.h>
#include <avr/pgmspace.h>

namespace srd {

class LCD
{
  static uint8_t bits[];
  static char scratch[12];
  void writeByte(uint8_t byte, uint8_t isData);
#ifdef LCD_WANT_SAMPLE
  void writeSample();
# if (LCD_COLS > 20)
  static PROGMEM const char defLine0[];
  static PROGMEM const char defLine1[];
# elif (LCD_COLS < 20)
  static PROGMEM const char defLine0[];
  static PROGMEM const char defLine1[];
# else
  static PROGMEM const char defLine0[];
  static PROGMEM const char defLine1[];
# endif
#endif

public:
  LCD(const Pin& clock, const Pin& data, const Pin& sync)
   : clock(clock)
   , data(data)
   , sync(sync) {
    init();
    };

  inline void clear() {
    writeCommand(0x01);
    };
  void beginLine(int8_t line = 1);
  void endLine();
  void configure(uint8_t useBits, uint8_t twoLines, uint8_t bigFont);
  void print(char* s);
  void print(int8_t v);
  void print(uint8_t v);
  void print(int16_t v);
  void print(uint16_t v);
  void print(int32_t v);
  void print(uint32_t v);
  void printLine(uint8_t line, PGM_P s);
  inline void writeCommand(uint8_t cmd) {
    writeByte(cmd, 0);
    if (cmd & 0x01 || cmd & 0x02) _delay_ms(16);
    };
  inline void writeData(uint8_t data) {
    writeByte(data, 1);
    ++pos;
    };

protected:
  void init();

private:
  Pin clock;
  Pin data;
  Pin sync;
  uint8_t pos;
  };
};
#endif
