/*
 * file:        Pin.h
 * purpose:     basic IO abstraction of a pin and port
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 *              based on the work of Karl-Heinz Buchegg
 *              http://www.mikrocontroller.net/topic/350418#3892395
 * copyright:   2014-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef PIN_H
#define PIN_H
#include "Base.h"
#include <avr/io.h>
#include <avr/interrupt.h>

namespace srd {

struct PortDesc
{
  volatile uint8_t* DdrReg;
  volatile uint8_t* PortReg;
  volatile uint8_t* PinReg;
  };


class Pin
{
public:
  Pin(const PortDesc& port, uint8_t pin)
   : port(port)
   , pin(pin) {
    }

  inline void toOutput() { *port.DdrReg  |=  (1 << pin); }
  inline void toInput()  { *port.PortReg &= ~(1 << pin);
                           *port.DdrReg  &= ~(1 << pin); }
  inline void setLow()   { cli(); *port.PortReg &= ~(1 << pin); sei(); }
  inline void setHigh()  { cli(); *port.PortReg |=  (1 << pin); sei(); }
  inline void toggle()   { if (isHigh()) setLow();
                           else          setHigh(); }

  inline int8_t isHigh() const { return   *port.PinReg & (1 << pin);  }
  inline int8_t isLow()  const { return !(*port.PinReg & (1 << pin)); }

  inline int8_t getPin() const { return pin; }

private:
  const PortDesc& port;
  const uint8_t   pin;
  };

EXTERN PortDesc PortA
#ifdef PLACE_GLOBALS_HERE
  = { &DDRA, &PORTA, &PINA }
#endif
  ;
EXTERN PortDesc PortB
#ifdef PLACE_GLOBALS_HERE
  = { &DDRB, &PORTB, &PINB }
#endif
  ;
EXTERN PortDesc PortC
#ifdef PLACE_GLOBALS_HERE
  = { &DDRC, &PORTC, &PINC }
#endif
  ;
EXTERN PortDesc PortD
#ifdef PLACE_GLOBALS_HERE
  = { &DDRD, &PORTD, &PIND }
#endif
  ;
};
#endif

