/*
 * file:        PushButton.h
 * purpose:     class for timebased interrupt (16bit timer).
 *              clients can be registered to be executed from interrupt
 *              default usage is to use the system interrupt with a millisecond as time base
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef SYSTEMTICKER_H
#define SYSTEMTICKER_H
#include "Base.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>

namespace srd {

class SystemTicker {
  static SystemTicker* ticker;
  static volatile uint32_t seconds;
  static volatile uint16_t ticks;
  static uint16_t maxTicks;

  static void run() __asm__(STR(TIMER1_OVF_vect)) __attribute__((__signal__, __used__, __externally_visible__));

  // setup the 16bit Timer as fast pwm timer with ICR1 as top
  // overflow-interrupt will be the system tick handler and execute the static function run()
  // timer frequency is set by interruptsPerSecond constructor parameter,
  // but the timer can be used as 16bit fast pwm timer any way.
  // PWM-output will be enabled and disabled by enablePwmOutput[AB]
  // Its not allowed to stop the timer or change prescaler settings.
  void setupTimer(uint16_t interruptsPerSecond);

public:
  SystemTicker(uint16_t interruptsPerSecond = 1000);

  void enablePwmOutputA(int8_t enable=1, int8_t inverted=0);
  void enablePwmOutputB(int8_t enable=1, int8_t inverted=0);

  inline void setToggleValueA(int16_t v) {
#ifdef __AVR_ATmega16__
    OCR1A = v;
#endif
    };

  inline void setToggleValueB(int16_t v) {
#ifdef __AVR_ATmega16__
    OCR1B = v;
#endif
    };
  inline uint32_t getSeconds() const { return seconds; }
  inline uint16_t getTicks() const   { return ticks;   }
  };                                                       // class Timer16Bit
};
#endif
