/*
 * file:        main.C
 * purpose:     sample for usage of framework
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2014-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#define PLACE_GLOBALS_HERE
#include "SystemTicker.h"
#include "Led.h"
#include "PushButton.h"
#include "Buzzer.h"
#include "LCD.h"
#include "ADC.h"
#include "UI.h"
#include <util/atomic.h>
#include <stdlib.h>

using namespace srd;

extern void lcdDisplay(LCD* lcd, uint32_t* secs, int8_t n, int16_t *v);


int main(void) {
  uint32_t oTime  = 0;
  uint16_t oTicks = 0;
  int16_t adcValues[8] = {0};
  uint8_t changed = 0;
  int8_t numConversions = 0;
  SystemTicker ticker;
  Buzzer buzzer(Pin(PortB, 3));
  LCD lcd(Pin(PortD, 6), Pin(PortD, 4), Pin(PortD, 5));
  LED ErrorLed(Pin(PortC, 2));
  LED StatusLed(Pin(PortC, 3));
  PushButton t1(Pin(PortD, 0));
  PushButton t2(Pin(PortD, 1));
  PushButton t3(Pin(PortD, 2));
  PushButton t4(Pin(PortD, 3));
  ADConverter adc(ADConverter::AVCC, ADConverter::PS_128);

  buzzer.doBeep();
  work.mode = MD_CONFIG;
  work.counter = 300;
  adc.setAutoTriggerMode(ADConverter::FreeRunning);
  adc.enableChannel(Pin(PortA, 0));
  adc.enableChannel(Pin(PortA, 1));
  adc.enableChannel(Pin(PortA, 2));
  adc.enableChannel(Pin(PortA, 3));
  adc.enableAutoTrigger();
  lcd.clear();

  for (;;)
      {
      // any time
      if (t1.pressedCount() > BUTTON_SAMPLING) {
         t1.reset();
         ++work.counter;
         changed = 1;
         }
      if (t2.pressedCount() > BUTTON_SAMPLING) {
         t2.reset();
         --work.counter;
         changed = 1;
         }
      if (t3.pressedCount() > BUTTON_SAMPLING) {
         t3.reset();
         ErrorLed.toggle();
         if (--work.mode < 0) work.mode = MD_SHOWTIME;
         changed = 1;
         }
      if (t4.pressedCount() > BUTTON_SAMPLING) {
         t4.reset();
         ErrorLed.toggle();
         if (++work.mode > MD_SHOWTIME) work.mode = MD_CONFIG;
         changed = 1;
         }

      if (numConversions != adc.conversionCompleted()) {
         numConversions = adc.conversionCompleted();
         for (int i=0; i < 8; ++i) {
             adcValues[i] = adc.getValue(i);
             }
         }

      // every millisecond
      if (oTicks != ticker.getTicks()) {
         ATOMIC_BLOCK(ATOMIC_FORCEON) {
                oTicks = ticker.getTicks();
                }
         buzzer.run();

         // button debouncing
         if (!(oTicks % IO_CHECK)) {
            t1.checkPressed();
            t2.checkPressed();
            t3.checkPressed();
            t4.checkPressed();
            }
         }

      // once a second
      if (oTime != ticker.getSeconds()) {
         ATOMIC_BLOCK(ATOMIC_FORCEON) {
                oTime = ticker.getSeconds();
                }
         buzzer.check4Repeat();

         if (work.mode == MD_COUNTDOWN && work.countDown > 0) {
            if (--work.countDown == 0) {
               work.mode = MD_DONE;
               buzzer.alarm();
               changed = 1;
               }
            }
         lcdDisplay(&lcd, &oTime, 8, adcValues);
         itoa(t1.pressedCount(), work.buf, 10);
         lcd.print(work.buf);
         }

      // async actions
      if (changed) {
         changed = 0;

         lcdDisplay(&lcd, &oTime, 8, adcValues);

         switch (work.mode) {
           case MD_LIGHT:
                ErrorLed.Off();
                break;

           case MD_COUNTDOWN:
                ErrorLed.Off();
                StatusLed.On();
                work.countDown = work.counter;
                break;

           case MD_DONE:
                adc.start();
                StatusLed.Off();
                break;

           case MD_SHOWTIME:
                adc.stop();
                ErrorLed.Off();
                adc.reset();
                break;
           }
         }
      }
  return 0;
  }
